<?php
/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// +-----------------------------------------------------------+
// | PHP version 4                                             |
// +-----------------------------------------------------------+
// | Copyright (c) Trond K. Pettersen @ http://lingo.uib.no    |
// | Any script (c) Lingo is subject to the GNU/GPL license.   |
// +-----------------------------------------------------------+
// | These scripts require that the PEAR base directory is in  |
// | your include_path. PEAR::DB is subject to the PHP license.|
// | See http://pear.php.net for help or more info on PEAR.    |
// +-----------------------------------------------------------+

if(!defined('MOO2DB')) {
  header("HTTP/1.1 403 Forbidden");
  exit("<h1>403 Forbidden</h1>");
}

require_once('MOODB.php');

class MOO 
{
  
  // Private stuff
  var $host;
  var $port;  
  var $parsed_url;
  
  // Public
  var $db;
  var $token; 
  var $moo_id; // DB id
  var $username;
  var $backroute;
  
  
  /**
       * Constructor
       *
       * @param id MOO id in DB
       */
  function MOO($moo_id=NULL) 
  {
    
    if(array_key_exists('back_route', $_POST)) {
      $this->set_backroute(urldecode($_POST['back_route']));
    }
    
    $this->db = new MOODB();
    
    if($moo_id)  {
      $this->set_from_db($moo_id);
    }
    
  }
  
  
  /**
       * Set back-route URL
       *
       * @param url string URL
       * @return void
       */
  function set_backroute($url)
  {
    $this->backroute = $url;
    $this->parsed_url = parse_url($url);
    $this->host = $this->parsed_url['host']; // shorcut
    $this->port = array_key_exists('port',$this->parsed_url) ? $this->parsed_url['port'] : 80; // shorcut
  }
  
  
  /** 
       * Appends param to back-route URL.
       * Assumes that the argument separator of back-route URL is ;
       *
       * @param name string arg.  name
       * @param value string arg. value
       *
       * @return void
       */
  function append_url_param($name,$value='')
  {
    if(strpos($this->backroute, '?') === false)  {
      $this->backroute .= '?' . urlencode($name) . '=' . urlencode($value); 
    } else {
      $this->backroute .= ';' . urlencode($name) . '=' . urlencode($value);
    } 
  }
  
  /**
       * Login MOO.
       *
       * If MOO's user credentials exists in POST-data, use default-login
       * else check the MOO's token.
       *
       * @return true on success, eles false
       */
  function auth() 
  {
    $this->db->connect();
    if(array_key_exists('moo_username', $_POST)) {
      return $this->login();
    } else {
      return $this->token_auth();
    }
  }
  
  
  /**
       * Check MOO-name and password from POST against 
       * data stored in database. Passwords in the database
       * are stored in the form of a SHA-1 hash.
       *
       * If login succeeds, create new token (if not token login)
       * and store token and back-route URL in DB.
       *
       * @return true on success, else false
       */
  function login()
  {
      
    $mooid   = array_key_exists('moo_username', $_POST) ? trim(urldecode($_POST['moo_username'])) : NULL;
    $moopass = array_key_exists('moo_password', $_POST) ? sha1(trim(urldecode($_POST['moo_password']))) : NULL;
    
    if(!$mooid || !$moopass) {
      return false;
    }
    
    $this->username = $mooid;
    $pt = $this->get_moo_data($this->username);
    
    if(empty($pt)) {
      // Empty or no array
      return false;
    }
    
    if($moopass == $pt['password'])  {      
      $this->moo_id = $pt['id'];
      $this->token  = $pt['token'];
      $this->store_moo_in_db();  // Store backroute and token in DB
      return true;  
    }
      
    return false;

  }
  
  
  
  /**
       * Check for moo_token in POST, and, if such exists, 
       * assume MOO to be logged in.
       *
       * @return true on success, else false.
       */
  function token_auth()
  {
    $moo_token = array_key_exists('moo_token', $_POST) ? trim(urldecode($_POST['moo_token'])) : NULL;
    
    if(!$moo_token)  {
      return false;
    }
    
    // If token exists in DB, assume OK MOO.
    $query = 'SELECT moo_id, backroute_url
          FROM '.$this->db->cfg['tbl_prefix'].'moo_data
          WHERE token='.DB_Common::quote($moo_token);
    
    $res = $this->db->executeQuery($query);
    $row = $this->db->fetchRow();
    
    //print_r($row);exit;
    
    if( is_array($row) && isset($row[0]) && (strlen($row[0]) >= 1) ) {
      $this->moo_id = $row[0];
      $this->backroute = $row[1];
      return true;
    }

    return false;
  }
  

  /** 
       * @return boolean true on success, else false 
       */
  function logout()
  {
    $lq = 'DELETE FROM '.$this->db->cfg['tbl_prefix'].'moo_data 
         WHERE moo_id = '.DB_Common::quote($this->moo_id);
    
    if($this->db->executeQuery($lq)) {
      $this->token = "0"; // Fool the browser (don't send "deleted" for cookie value) and MOO (0 vs. "")
      return true;
    }
    
    return true;
  }
  
  
  /** 
       * @return array string moo id and token 
       */
  function get_moo_data($username)
  {
  
    $moo_data = array();
  
    $query = 'SELECT m.id, t.token, m.password_hash
          FROM '.$this->db->cfg['tbl_prefix'].'moo m
            LEFT JOIN '.$this->db->cfg['tbl_prefix'].'moo_data t ON m.id = t.moo_id
          WHERE m.username='.DB_Common::quote($username);
    
    $res =& $this->db->executeQuery($query);
    
    if( $row =& $this->db->fetchRow() ) {
      $moo_data['password'] = $row[2];
      $moo_data['id'] = $row[0];
      $moo_data['token'] = $row[1];
    }
    
    return $moo_data;

  }
  
  
  /**
       * Generates new token, and inserts it into the DB.
       *
       * @return true on success, else false.
       */
  function store_moo_in_db()
  {
    
    srand(99999);
    $newtoken = sha1(microtime() . md5(rand(1000, 10000)));
    
    if($this->token) {
      // Update
      $query = 'UPDATE '.$this->db->cfg['tbl_prefix'].'moo_data
                SET token = ' . DB_Common::quote($newtoken) . ',
                backroute_url = ' . DB_Common::quote($this->backroute) . ', 
              created = NOW()
            WHERE moo_id = ' . DB_Common::quote($this->moo_id);
    } else {
      // Insert / "Login"
      $query = 'INSERT INTO '.$this->db->cfg['tbl_prefix'].'moo_data (moo_id, token, backroute_url)
                VALUES (' . DB_Common::quote($this->moo_id) . ', ' .
                    DB_Common::quote($newtoken) .', ' .
                  DB_Common::quote($this->backroute) .')';
    }
    
    if($this->db->executeQuery($query)) {
      $this->token = $newtoken;
      return true;
    }
    
    return false;
    
  }
  
  
  /**
       * @param $id int 
       * @return true on success, else false
       */
  function set_from_db($id)
  {
    $query = 'SELECT m.username, t.backroute_url, t.token
              FROM '.$this->db->cfg['tbl_prefix'].'moo_data t, 
               '.$this->db->cfg['tbl_prefix'].'moo m
          WHERE m.id = t.moo_id 
            AND m.id = '.((int)$id);

    $res = $this->db->executeQuery($query);

    if(!$res) {
      return false;
    }
    
    $row = $this->db->fetchRow();
    
    if($row) {
      $this->moo_id = (int)$id;
      $this->username = $row[0];
      $this->set_backroute($row[1]);
      $this->token = $row[2];
      return true;
    }
    
    return false;
    
  }
  
    
}

?>