<?php
/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// +-----------------------------------------------------------+
// | PHP version 4                                             |
// +-----------------------------------------------------------+
// | Copyright (c) Trond K. Pettersen @ http://lingo.uib.no    |
// | Any script (c) Lingo is subject to the GNU/GPL license.   |
// +-----------------------------------------------------------+
// | These scripts require that the PEAR base directory is in  |
// | your include_path. PEAR::DB is subject to the PHP license.|
// | See http://pear.php.net for help or more info on PEAR.    |
// +-----------------------------------------------------------+

// Check to ensure that script is not called 'individually'.
if(!defined('MOO2DB')) {
  header('HTTP/1.1 403 Forbidden');
  exit('<h1>403 Forbidden</h1>');
}

// The PEAR base directory must be in your include_path
if(!(@include_once('DB.php'))) {
  header('HTTP/1.1 500 Internal Server Error');
  exit('<h1>500 Internal Server Error</h1>'.
       '<p>Failed to include PEAR::DB</p>');
}


class MOODB extends DB
{
  
  // public
  var $dsn;
  var $conn;
  var $cfg;
  
  // private
  var $res;
  
  
  /** Constructor */  
  function MOODB()
  {
    global $config;
    $this->cfg =& $config['db'];
    $this->show_err = (bool)$this->cfg['display_errors'];
    $this->dsn = $this->cfg['type'].'://'.$this->cfg['user'].':'.
                 $this->cfg['pass'].'@'.$this->cfg['host'].':'.
                 $this->cfg['port'].'/'.$this->cfg['schema'];
  }
  
  
  /**
       * @return boolean true upon success, else false.
       */
  function connect()
  {
    $this->conn =& parent::connect($this->dsn);
    if(!parent::isError($this->conn)) {
      return true;
    } else {
      return false;
    }
  }
  
  function close()
  {
    $this->conn->disconnect();
  }
  
  
  /**
       * @param String Database query to be executed
       *
       * @return boolean true upon success, else false.
       *
       * @access private
       */
  function executeQuery($q)
  {
  
    if(!$this->conn) {
      $this->connect();
    }
  
    if(is_a($this->conn, 'DB_Error')) {
      return false;
    }
    
        $this->res =& $this->conn->query($q);
    
        if(!(parent::isError($this->res))) {
      return true;
      }
    
    return false;

  }
  
  /**
      * @return result row from DB
      */
  function fetchRow()
  {
    if(is_a($this->res, 'DB_Error') || (!$row = $this->res->fetchRow())) {
      return array();
    }
    return $row;
  }
    
    
  /**
       * Generates error from DB::DB_Error objects if
       * the member-var. $show_err is set to true.
       *
       * @return string Errors
       */
  function getErrors()
  {
  
    if(!$this->show_err) {
      return '';
    }
    
    if($this->conn && method_exists($this->conn, 'getMessage')) {
      $eobj =& $this->conn;
    } elseif($this->res && method_exists($this->res, 'getMessage')) {
        $eobj =& $this->res; 
    } else {
      return 'A database related error occurred';
    }
    
    return 'Standard Message: '   . $eobj->getMessage()   . "\n".
           'Standard Code: '      . $eobj->getCode()      . "\n".
           'DBMS/User Message: '  . $eobj->getUserInfo()  . "\n".
           'DBMS/Debug Message: ' . $eobj->getDebugInfo() . "\n";
  }

}

?>