<?php
/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// +-----------------------------------------------------------+
// | PHP version 4                                             |
// +-----------------------------------------------------------+
// | Copyright (c) Trond K. Pettersen @ http://lingo.uib.no    |
// | Any script (c) Lingo is subject to the GNU/GPL license.   |
// +-----------------------------------------------------------+
// | These scripts require that the PEAR base directory is in  |
// | your include_path. PEAR::DB is subject to the PHP license.|
// | See http://pear.php.net for help or more info on PEAR.    |
// +-----------------------------------------------------------+

/**
 * This class represents the MOO2DB MOO user. 
 *
 * Although it would've been nice to use sessions for
 * temporary data storage (e.g. tokens, etc.), it does currently not. 
 * Some strange browser behaviour was detected during testing; 
 * the MOOs seem to receive the session cookie
 * from some browsers, which may cause the user to be logged
 * out of the MOO. This ought to be fixed in later versions, so that
 * we don't have to rely on $_POST-vars and hidden HTML form fields alone...
 */

// Check to ensure that script is not called "individually".
if(!defined('MOO2DB')) {
  header("HTTP/1.1 403 Forbidden");
  exit("<h1>403 Forbidden</h1>");
}


require_once('File.php');
require_once('Language.php');


class User 
{

  // ________________________ Static methods ________________________ \\
  
  /**
     * Builds a list of a given user's files.
     * Calculates total filesize in bytes.
     *
     * @param usrnr   int user number (in MOO)
     * @param sort_by string name of sort function
     *
     * @return array mixed
     */
  function get_files($usrnr, $sort_by='')
  {
  
    global $config;
  
    $totsize    = 0;
    $file_names = array();
    $files      = array();
        
    $filedir = $config['upload_path'] . '/' . $usrnr;  
    
    // Create list of files in folder
    if ($handle = opendir($filedir)) {
      while (false !== ($file = readdir($handle))) {
         if(($file != 'index.php') && ($file != '.') && ($file != '..'))  {  
           $totsize += filesize($filedir.'/'.$file); // Add file size to total
          $files[$file] = new File($filedir . '/' . $file, $usrnr); // Add file obj. to list
        }
      }
       closedir($handle);
    }
    
    // We should've sorted on insertion instead of this, but it ought to be fast
    if($sort_by == 'size') {
      usort($files, array('File', 'sort_size'));
    } elseif ($sort_by == 'size-desc') {
      usort($files, array('File', 'sort_size_desc'));
    } elseif($sort_by == 'filename') {
      usort($files, array('File', 'sort_filename'));
    } elseif ($sort_by == 'filename-desc') {
      usort($files, array('File', 'sort_filename_desc'));
    } elseif ($sort_by == 'modified-desc') {
       usort($files, array('File', 'sort_modified_desc'));    
    } else {
      usort($files, array('File', 'sort_modified'));
    }
    
    return array('files'=>$files, 'size'=>$totsize);

  }
  
  

  // ________________________ Member vars. ________________________ \\

  // Private
  var $authenticated; // Boolean auth. flag
  
  // Public  
  var $user_nr;               // Int user # in moo
  var $upload_magic_string; // String token from MOO
  var $fetch_css;           // String URL to MOO-side CSS
  var $fetch_strings;       // String URL to MOO-side strings
  var $lang;                // Object Language 
  var $action;         // String Action (upload/delete/view)
  var $moo;             // Object MOO
  var $errors;         // Array String Errors suited for user display

  var $filefolder;     // String path to user's folder on server
  var $files;           // Array Object Files
  var $file_names;     // Array String file names (short-cut)
  var $tot_file_size; // Int file size in bytes  (short-cut)
  var $file_location; // URL to last uploaded file
  var $temp_token;    // Token for MOO, the MOO needs this to accept in-MOO changes
  
  var $deleted_files; // Array URLs to deleted files
  
  
  // ________________________ Methods ________________________\\
  
  
  // Constructor
  function User($sessid=null)
  {
    // Init. values
    $this->temp_token = '';
    $this->authenticated = false;
    $this->moo = new MOO();
    $this->user_nr = 1;
    $this->tot_file_size = 0;
    $this->file_names = array();
    $this->errors = array();
    $this->deleted_files = array();
    $this->set_action();
    $this->file_location = null; // No file uploaded
    
    // Do authentication
    if($sessid) {
      $this->auth($sessid);
    }
    
  }  
  
  /** 
   * If upload_magic_string is received, and the session with id=upload_magic_string has
   * been started earlier - and the user is granted permission - user is
   * assumed to be OK.
   *
   * @return void
   */
  function auth($sessid) 
  {
    
    global $config;
    
    $userdata = $this->get_user_data($sessid);
    if(empty($userdata) || !array_key_exists('user_nr', $userdata)){
      // Authentication failed
      return false;
    }
    
    $this->upload_magic_string = $sessid;
    $this->user_nr = (int)$userdata['user_nr'];
    $this->fetch_css = (string)$userdata['fetch_css'];
    $this->fetch_strings = (string)$userdata['fetch_strings'];
    $this->moo->set_from_db($userdata['moo_id']);
    $this->filefolder = $config['upload_path'] . '/' . $this->user_nr;
    $this->authenticated = true;
  
    $this->set_language();
    
  }
  
  
  /** @return true if user has been authenticated, else false. */
  function is_authenticated()
  {
    return ($this->authenticated === true) ? true : false;
  }
  
  
  // _____________________ Getters ____________________ \\  
  
  
 /**
     * Fetch user data from DB based on upload_magic_string
     *
     * @param ut string user's token
     */
  function get_user_data($ut)
  {
    
    $usrtkn = trim(((string)$ut));
    if(empty($usrtkn)) {
      return false;
    }
    
    if(!$this->moo->db->conn) {
      $this->moo->db->connect();
    }
    
    $q = 'SELECT moo_id, user_nr, fetch_css, fetch_strings
          FROM ' . $this->moo->db->cfg['tbl_prefix'] . 'user
          WHERE upload_magic_string = ' . DB_Common::quote($usrtkn);
    
    $res = $this->moo->db->executeQuery($q);
      
    if($res && ($row = $this->moo->db->fetchRow())) {
      return array('moo_id'=>$row[0],
                   'user_nr'=>$row[1],
                   'fetch_css'=>$row[2],
                   'fetch_strings'=>$row[3]);
    }
    
    return array();
    
  }
  
  
 /**
     * @return string name of uploaded file (note; only 1 file), used for saving
     */
  function get_upload_filename() {
    return str_replace(' ', '_', basename($_FILES['file']['name']));
  }
  
 /**
     * @return size of uploaded file in bytes
     */
  function get_upload_filesize() {
    return $_FILES['file']['size'];
  }
  
  /** @return array files */
  function get_file_list()
  {
    return $this->files;
  }
  
 /**
     * @return string
     */
  function get_space_left() {
   
    global $config;
   
    $max  = $config['max_upload_size']*1024*1024;
    $size = File::get_file_size($max - $this->tot_file_size);
   
    if($this->tot_file_size > $max) {
      return '-'.$size;
    } else {
      return $size;
    }
    
  }
  
  /** @return string location of uploaded file */
  function get_file_location()
  {
    return $this->file_location;
  }
  
  
  /** 
       * This performs a simple filtering on the CSS from MOO.
       * Attributes are not removed, however, but since the fetch_css-URL
       * SHOULD come from the MOO we haven't written a custom 
       * function to filter those out (seems unnecessary)...but add 
       * at will.
       *
       * @return string CSS to include in output
       */
  function get_moo_css() {
    
    if( strlen($this->fetch_css) &&
        ini_get('allow_url_fopen') && 
        ($f = file_get_contents($this->fetch_css))
    ){
      return strip_tags($f, '<link><style>');
    }
    
    return '';
    
  }

  
  // _____________________ Setters ____________________ \\
  
  
  /**
       * Set user language.
       * 
       * @return void
       */
  function set_language()
  {
  
    if($this->fetch_strings != '') {
      $this->lang =& new Language($this->fetch_strings);
    } else {
      $this->lang =& new Language();
    }
    
    $this->lang->parse();
    
    if($this->fetch_strings && empty($this->lang->strings)) {
      // Re-try w/default language
      $this->errors = array_merge($this->errors, $this->lang->errors);
      $this->lang =& new Language();
      $this->lang->parse();
    }
    
  }
  
    
  
  // User action
  function set_action()
  {
    $a = 'view';
    
    if(array_key_exists('file', $_FILES) && !array_key_exists('update', $_POST)) {
      $a = 'upload';
    } elseif( array_key_exists('delete', $_POST) ) {
      $a = 'delete'; 
    } elseif( array_key_exists('update', $_GET) ||
              array_key_exists('update', $_POST)) {
      $a = 'update';
    }
    
    $this->action = $a;
  }
  
  
  /**
       * If user folder does not exist -> create folder and generate file list
       * else -> generate file list
       *
       * @return void
       */  
  function set_folder_and_files($sort_by='modified')
  {
  
    if(!$this->filefolder) {
      return;
    }
    
    global $config;
    
    $this->file_names = array();
    $this->files = array();
    
    if(!is_dir($this->filefolder))  {
      mkdir($this->filefolder, 0755);
      // Create index-file. 
      // NB! This is not something that makes the folder in-accessible to "lurkers"
      $fp = @fopen($this->filefolder.'/index.php', 'w+');
      @fwrite($fp, sprintf(file_get_contents(dirname(__FILE__).'/../templates/index.php'), 
             $this->moo->backroute));
      @fclose($fp);
    }
    
    $files_info = User::get_files($this->user_nr, $sort_by);
    
    $this->files = $files_info['files'];
    
    foreach($files_info['files'] as $fil) {
      $this->file_names[] = $fil->get_filename(); // Short-cut
    }
    
    $this->tot_file_size = $files_info['size'];
      
  }
  
  
  /** 
       * @param s string error
       *
       * @return void
       */
  function set_error($s)
  {
    $e = (string)trim($s);
    if(strlen($e) > 0) {
      $this->errors[] = $e;
    } else {
      $this->errors[] = $this->lang->strings['err_unknown'];
    }
  }
  
  
  // _____________________ Utils ____________________ \\
  
  
  /** 
       * @return true if file uploaded OK, else false 
       */
  function do_upload()
  {
  
    global $config;
    
    // We need this done
    $this->set_folder_and_files();
    
    if(!array_key_exists('file', $_FILES) || ($_FILES['file']['size'] == 0)) {
      // No file received
      $this->set_error(sprintf($this->lang->strings['err_no_file_received'], 
                       @ini_get('upload_max_filesize')));
      return false;
    }
    
    $fname = $this->get_upload_filename();
      
    if( !empty($this->file_names) && in_array($fname, $this->file_names) ) {
      // Already existing file. Delete first.
      $this->set_error($this->lang->strings['err_existing_file'] . ': ' . $fname);
      return false;
    }
    
    if(!$this->check_filename($fname)) {
      return false;
    }
    
    
    $newsize = $this->tot_file_size + $_FILES['file']['size'];
    if($newsize > ($config['max_upload_size']*1024*1024)) {
      // Upload will exceed quota
      $this->set_error( sprintf($this->lang->strings['err_quota'], 
                File::get_file_size($config['max_upload_size']*1024*1024), 
                File::get_file_size($this->tot_file_size), 
                File::get_file_size($_FILES['file']['size'])) );
      return false;
    }    
    
    return $this->save_file($fname);
    
  }
  
  
  /**
       * @param name uploaded file should be saved as
       * @return true on success, else false
       */
  function save_file($fname) 
  {
    global $config;
    
    // The file must be reachable by every browser 
    // (not just MSIE which e.g. accepts white space).
    $path2file = $this->filefolder . "/" . htmlspecialchars($fname);
    
    if(move_uploaded_file($_FILES['file']['tmp_name'], $path2file)) {
      $this->file_location = $config['upload_folder'] . '/' . 
                             $this->user_nr . '/' . $fname;
      // Rebuild file list
      $this->set_folder_and_files();
      return true;
    }
    
    return false;
    
  }
  
  /**
       * Replaces current file with new file
       *
       * @param old file name of old file
       * @return true on success, else false
       */
  function update_file($old) 
  {
   
    global $config;
    
    $fname = $this->get_upload_filename();
    
    if(strcmp($old, $fname) != 0) { 
      // We compare the file names because many users seem to get confused 
      // and don't understand what happens if we let them update an existing 
      // file with a new but differently named file
      $this->set_error(sprintf($this->lang->strings['err_update_ineq'], $old));
      return false;
    }
    
    if(!$this->check_filename($fname)) {
      // Just an extra check in case someone tries to screw us, somehow
      return false;
    }
    
    $path2file = $config['upload_path'].'/'.$this->user_nr . '/' . $fname;
    if(!file_exists($path2file)) {
      $this->set_error($this->lang->strings['err_update_nonexisting']);
      return false;
    }
       
    if($this->tot_file_size <= 0) {
      $this->set_folder_and_files();
    }
    
    $new_size = $this->tot_file_size - filesize($path2file) + 
                $this->get_upload_filesize();
    
    if($new_size > ($config['max_upload_size']*1024*1024)) {
      // New size exeeds limit
      $this->set_error($this->lang->strings['err_max_file_size']);
      return false;
    }
    
    // remove existing file
    if(!unlink($path2file)) {
      $this->set_error(sprintf($this->lang->strings['err_update_unlink'], 
                                htmlentities($fname)));
      return false;
    }
    
    return $this->save_file($fname);
    
  }
  
  
  /**
       * @param array name of files to be deleted
       * @return true if all files deleted OK, else false
       */
  function delete_files($delarr)
  {
  
    global $config;
      
    settype($delarr, 'array');  // In case only 1 file is selected
    
    if(empty($delarr)) {
      $this->set_error($this->lang->strings['err_no_file_selected']);
      return false;
    }
    
    $this->set_folder_and_files();
    
    $totres = true;
    foreach($delarr as $delfile) {
      if( file_exists($this->filefolder . '/' . $delfile) && 
          in_array($delfile, $this->file_names))
      {
        $res = unlink($this->filefolder . '/' . $delfile);
        if(!$res) {
          $this->set_error($this->lang->strings['err_delete_failed'] . ' ' . 
                                                      htmlentities($delfile));
          $totres = false;
        } else {
          // Add file to list of deleted files
          if(!is_array($this->deleted_files)) {
            $this->deleted_files = array();
          }
          array_push($this->deleted_files, $config['upload_folder'] . '/' . 
                                           $this->user_nr . '/' . $delfile);
        }
      }
    }
        
    $this->set_folder_and_files();
    
    return $totres;  
      
  }

  
  /**
       * @param string file name
       * @return true if file name passes all tests
       */
  function check_filename($fname)
  {
    global $config;
    
    if(!File::is_valid_filetype($fname)) {
      // Invalid file type (extension).
      $this->set_error( $this->lang->strings['err_invalid_type'] . 
                        ' ( ' . implode(', ', $config['file_extensions']) . ' ) ' );
      return false;
    }
        
    if(!File::is_valid_filename($fname)) {
      // Invalid file name
      $this->set_error($this->lang->strings['err_invalid_name']);
      return false;
    }
    
    return true;
    
  }
  
  
  /**
       * Renews the token in the DB
       *
       * Not necessary as upload_magic_string is used to auth. user
       * if one token gets stolen / sniffed, it's likely that so is the other.
       */
  function renew_form_token()
  {
    srand(101);
    $this->form_token = sha1(md5(microtime().rand(1, 100)));
    $q = 'UPDATE ' . $this->moo->db->cfg['tbl_prefix'] . 'user
          SET form_token = ' . DB_Common::quote($this->form_token) . '
          WHERE user_nr = \'' . ((int)$this->user_nr) . '\'';
    return $this->moo->db->executeQuery($q);
  }
  
    
  /**
       * Re-route user -> back to MOO
       */
  function send_back()
  {
    if($this->moo->backroute && !headers_sent()) {
      header('Location: '.$this->moo->backroute);
      exit;
    } elseif(headers_sent()) {
      $this->set_error('Tried to send back, but headers were already sent.');
    } else {
      $this->set_error($this->lang->strings['err_backroute']);
    }
  }
  
}


?>