<?php
/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// +-----------------------------------------------------------+
// | PHP version 4                                             |
// +-----------------------------------------------------------+
// | Copyright (c) Trond K. Pettersen @ http://lingo.uib.no    |
// | Any script (c) Lingo is subject to the GNU/GPL license.   |
// +-----------------------------------------------------------+
// | These scripts require that the PEAR base directory is in  |
// | your include_path. PEAR::DB is subject to the PHP license.|
// | See http://pear.php.net for help or more info on PEAR.    |
// +-----------------------------------------------------------+

/**
  * This script is the main PHP-side of the $uploader-script.
  * Users can view, upload and update files through this script.
  * Whenever a user has uploaded a file, he/she is re-routed back to the MOO.
  *
  * TODO: When file upload fails (e.g. illegal extension),
  *              let user try again instead of sending directly back.
  */

define('MOO2DB', '1.0rc1');

require_once('./includes/common.php');
require_once('./includes/classes/MOO.php');
require_once('./includes/classes/User.php');

// Get upload_magic_string :-D
if( array_key_exists('upload_magic_string', $_POST )) {
  // Standard auth.
  $magic_string = $_POST['upload_magic_string'];

} elseif( array_key_exists('upload_magic_string', $_GET) &&
          array_key_exists('update', $_GET)) {
  // Via GET on updata only
  $magic_string = $_GET['upload_magic_string'];

} else {
  $magic_string = NULL;
}

$user = new User($magic_string);

// DO NOT MESS WITH THE LINE BELOW
if($user->is_authenticated() !== true) {
  
  $user->set_language();
  
  if(!empty($user->moo->backroute)) {
  
    $user->moo->append_url_param('status', 'ERROR_NOT_AUTH');
    $user->send_back();

  } else {
    
    $user->set_error($user->lang->strings['err_auth_failed']);
    $user->set_error($user->lang->strings['err_aborted']);
    
    include_once('./includes/templates/header.php');
    include_once('./includes/templates/errors.php');
    include_once('./includes/templates/footer.php');
    
  }
  
  // User is not authenticated -> abort mission
  exit;

}


// _____________ User is authenticated ________________ \\


if(array_key_exists('temp_token', $_POST)) {
  $user->temp_token = $_POST['temp_token'];  
} elseif(array_key_exists('temp_token', $_GET)) {
  $user->temp_token = $_GET['temp_token'];
}


// ____________  Upload file. ____________ \\


if($user->action == 'upload') {

  $user->moo->db->close();

  if($user->do_upload() == true) {
  
    // File uploaded OK
    $user->moo->append_url_param('result', 'created_ok');
    $user->moo->append_url_param('number_of_files', count($user->file_names));
    $user->moo->append_url_param('tot_file_size', $user->tot_file_size); 
    $user->moo->append_url_param('temp_token', @$_POST['temp_token']);
    $user->moo->append_url_param('path',urlencode($user->get_file_location()));
    $user->send_back();
    
  } else {
  
    // Upload failed
    // Current action: send user back
    // Replace with; error message + try again.
    $user->moo->append_url_param('result', 'create_failed');
        $user->moo->append_url_param('explanation', implode('+', $user->errors));
    $user->send_back();
    
  }
  
  exit; // just in case

}


// _____________ Start output ______________ \\


// This could use some tidying up...

include_once('./includes/templates/header.php');


// ____________  Non-upload actions ____________ \\

if($user->action == 'delete') {

  // User wants to delete file(s)
  if( array_key_exists('confirm_delete', $_POST) && 
      $_POST['confirm_delete'] == 1) {

    // User has confirmed deletion
    $user->delete_files($_POST['delete']);
    if(!empty($user->deleted_files)) {
      // At least some files were deleted OK
      include_once('./includes/templates/delete.php');   
    }
  
  }
  
} elseif($user->action == 'update') {
  
  if(array_key_exists('update', $_POST)) {
    if(!$user->update_file($_POST['update'])) {
      // Update failed
      $user->set_error($user->lang->strings['err_update']);
    } 
  }
    
}


// If already done (upload/delete); rebuild.
if( array_key_exists('sort_by', $_POST) && !empty($_POST['sort_by'])) {
 
  $sort_by = $_POST['sort_by'];

} else {

  $sort_by = 'modified-desc';

}

$user->set_folder_and_files($sort_by);

// In case something went wrong...notify user.
include_once('./includes/templates/errors.php');

if($user->action == 'update') { 
  include_once('./includes/templates/update.php');  
}

if( $user->action != 'update' && 
   (($user->action != 'delete') || 
   ($user->action == 'delete' && empty($user->deleted_files)))) {
  include_once('./includes/templates/file_list.php'); // print file list
}

include_once('./includes/templates/footer.php');


$user->moo->db->close();

?>