<?php
/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// +-----------------------------------------------------------+
// | PHP version 4                                             |
// +-----------------------------------------------------------+
// | Copyright (c) Trond K. Pettersen @ http://lingo.uib.no    |
// | Any script (c) Lingo is subject to the GNU/GPL license.   |
// +-----------------------------------------------------------+
// | These scripts require that the PEAR base directory is in  |
// | your include_path. PEAR::DB is subject to the PHP license.|
// | See http://pear.php.net for help or more info on PEAR.    |
// +-----------------------------------------------------------+

/**
  * This script is used to authenticate MOO users.
  * It is "called" by the MOO's $uploader-object through a HTTP POST Request. 
  * 
  * Users may be added to, updated or deleted from the database.
  * (this is of course not the same DB as the MOO database)
  */

define('MOO2DB', '1.0rc1');

require_once('./includes/common.php');
require_once('./includes/classes/UserUpdater.php');

// Which action to take (add || remove user)
$_lacts = array('update', 'remove', 'get_file_info');
$action = (array_key_exists('action', $_POST) && in_array($_POST['action'], $_lacts)) ? $_POST['action'] : NULL;

if(!$action) {
  sendHttpHeader(406);
} 

// NB! User-input stored in $upload is not to be considered "clean" (yet).
$user_nr       = array_key_exists('user_nr', $_POST)       ? (int)$_POST['user_nr']                   : 0;
$moo_token     = array_key_exists('moo_token', $_POST)     ? trim(urldecode($_POST['moo_token']))     : '';
$fetch_css     = array_key_exists('fetch_css', $_POST)     ? trim(urldecode($_POST['fetch_css']))     : '';
$fetch_strings = array_key_exists('fetch_strings', $_POST) ? trim(urldecode($_POST['fetch_strings'])) : '';

if(array_key_exists('upload_magic_string', $_POST)) {
  $upload_magic_string = trim(urldecode($_POST['upload_magic_string']));
} else {
  $upload_magic_string = '';
}



// Always required data
if($user_nr <= 0 || $moo_token == '') {
    sendCookie('ERROR', 'ERROR_INCOMPLETE_DATA');
    sendHttpHeader(406);
}

// Magic string required on update
if( $action == 'update' && $upload_magic_string == '' ) {
  sendCookie('ERROR', 'ERROR_MAGIC_STRING');
  sendHttpHeader(406);
} 

if($action == 'get_file_info') {
  // Get file info. on given User
  $moo = new MOO();
  if($moo->auth() !== true) {
    // Which MOO???
    $moo->db->close();
    sendCookie('ERROR_AUTH_MOO');
    sendHttpHeader(403, 'HTTP/1.1 403 Forbidden Fruit');
  } else {
    $moo->db->close();
    require_once('./includes/classes/User.php');
    $finfo = User::get_files($user_nr);
    sendCookie('number_of_files', count($finfo['files']));
    sendCookie('total_file_size', $finfo['size']);
    sendHttpHeader(200, 'OK');
  }
  exit;
}



$userUpdater =& new UserUpdater( $user_nr, $upload_magic_string, $fetch_css, $fetch_strings );

if($userUpdater->moo->db->connect() !== true) {
    sendCookie('ERROR', 'ERROR_DB_CONNECT');
    sendHttpHeader(500, 'ISE');
}

// MOO authenticated? Legal MOO?
if( $userUpdater->moo->auth() !== true ) {
  $userUpdater->moo->db->close();
    sendCookie('ERROR', 'ERROR_AUTH_MOO');
    sendHttpHeader(403, '403 Forbidden. MOO Auth. failed.');
}

if($action == 'update') {
  // Update data on user
  $res = $userUpdater->updateUser();
} else {
  // Remove data on user
  $res = $userUpdater->removeUser();
}

// Close DB Conn.
$userUpdater->moo->db->close();

if( $res ) {
  // Database updated
  sendCookie($action, $userUpdater->user_nr);
    sendHttpHeader(200, 'db_updated');
} else {
  // Something went wrong!
    sendCookie('ERROR', 'ERROR_SERVER');
    sendHttpHeader(500, 'INTERNAL_SERVER_ERROR'."\n\n".$userUpdater->moo->db->getErrors());
}

?>